package com.jiuqi.script.rhino;

import java.util.Date;
import com.jiuqi.script.IScriptObject;
import java.util.Calendar;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.IdFunctionObject;
import org.mozilla.javascript.IdScriptableObject;
import org.mozilla.javascript.NativeJavaObject;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class ScriptHelper extends ScriptableObject
{
    private Scriptable scope;
    
    ScriptHelper(final Scriptable scope) {
        super.setParentScope(this.scope = scope);
    }
    
    public Object toJava(final Object obj) {
        return _toJava(obj, this.scope);
    }
    
    public Object toScript(final Object obj) {
        return _toScript(obj, this.scope);
    }
    
    public void print(final Object obj) {
        System.out.println(obj);
    }
    
    public String getClassName() {
        return "ScriptUtil";
    }
    
    static Object _toJava(final Object obj, final Scriptable scope) {
        if (obj == null) {
            return null;
        }
        if (obj instanceof RhinoScriptObject) {
            return ((RhinoScriptObject)obj).getScriptObject();
        }
        if (obj instanceof NativeJavaObject) {
            final Object o = ((NativeJavaObject)obj).unwrap();
            if (o instanceof RhinoScriptObject) {
                return ((RhinoScriptObject)o).getScriptObject();
            }
            return o;
        }
        else {
            if (!(obj instanceof IdScriptableObject)) {
                return obj;
            }
            final IdScriptableObject result = (IdScriptableObject)obj;
            if (result.getClassName().equals("Date")) {
                final IdFunctionObject v = (IdFunctionObject)ScriptableObject.getProperty((Scriptable)result, "getTime");
                final Object m = result.execIdCall(v, Context.getCurrentContext(), scope, (Scriptable)result, (Object[])null);
                final long millis = ((Double)m).longValue();
                final Calendar c = Calendar.getInstance();
                c.setTimeInMillis(millis);
                return c;
            }
            return result;
        }
    }
    
    static Object _toScript(final Object obj, final Scriptable scope) {
        if (obj == null) {
            return null;
        }
        if (obj instanceof IScriptObject) {
            return new RhinoScriptObject((IScriptObject)obj);
        }
        if (obj instanceof IdScriptableObject) {
            return obj;
        }
        if (obj instanceof NativeJavaObject) {
            final Object result = ((NativeJavaObject)obj).unwrap();
            if (result instanceof IScriptObject) {
                return new RhinoScriptObject((IScriptObject)result);
            }
            if (result instanceof Calendar) {
                final Calendar c = (Calendar)result;
                return Context.getCurrentContext().newObject(scope, "Date", new Object[] { new Long(c.getTimeInMillis()) });
            }
            if (result instanceof Date) {
                final Date d = (Date)result;
                return Context.getCurrentContext().newObject(scope, "Date", new Object[] { new Long(d.getTime()) });
            }
        }
        return obj;
    }
}
